import java.util.ArrayList;

/**
 * PrimeUtil
 * 
 * @author Taylor Born
 * @version September 2013
 */
public abstract class PrimeUtil 
{
    private static ArrayList<Integer> primes = new ArrayList<Integer>(){{ add(2); add(3); }};

    /**
     * @return Whether the given number is prime.
     */
    public static boolean isPrime(int n)
    {
        if (n <= 1)
            return false;
        if (n > primes.get(primes.size() - 1))
            calculateMoreFor(new LargerThanCondition(n));
        
        // Binary Search
        
        int low = 0;
        int high = primes.size() - 1;
        while (true)
        {
            int m = (low + high) / 2;
            if (primes.get(m) == n)
                return true;
            if (primes.get(m) < n)
                low = m + 1;
            else
                high = m - 1;
            if (low > high)
                return false;
        }
    }
    
    private static void calculateMoreFor(Condition c)
    {
        for (int last = primes.get(primes.size() - 1) + 2; c.keepGoing(); last++)
            for (int i = 0; i < primes.size(); i++)
                if (last % primes.get(i) == 0)
                    break;
                else if (i == primes.size() - 1)
                    primes.add(last);
    }
    
    public static void printPrimes()
    {
        for (Integer prime : primes)
            System.out.println(prime);
    }
    
    /**
     * @return The nth prime.
     */
    public static int getPrime(int nth)
    {
        if (nth - 1 > primes.size() - 1)
            calculateMoreFor(new NumberOfPrimesCondition(nth - 1));
        return primes.get(nth - 1);
    }
    
    /**
     * Prime Counting Function.
     * @return The number of primes less than or equal to value.
     */
    public static int pi(double d)
    {
        if (d > primes.get(primes.size() - 1))
            calculateMoreFor(new LargerThanCondition((int)d + 1));
        int low = 0;
        int high = primes.size() - 1;
        while (true)
        {
            int m = (low + high) / 2;
            if (primes.get(m) == d)
                return m + 1;
            if (primes.get(m) < d)
                low = m + 1;
            else
                high = m - 1;
            if (low > high)
            {
                if (d >= primes.get(m))
                    return m + 1;
                return m;
            }
        }
    }
    
    
    static abstract class Condition
    {
        public abstract boolean keepGoing();
    }
    static class LargerThanCondition extends Condition
    {
        private int num;
        
        public LargerThanCondition(int n)
        {
            num = n;
        }
        public boolean keepGoing()
        {
            return primes.get(primes.size() - 1) < num;
        }
    }
    static class NumberOfPrimesCondition extends Condition
    {
        private int num;
        
        public NumberOfPrimesCondition(int n)
        {
            num = n;
        }
        public boolean keepGoing()
        {
            return primes.size() - 1 < num;
        }
    }
}